package com.bohuikeji.delivery.backstage.entity;

import com.alibaba.fastjson.annotation.JSONField;
import com.baomidou.mybatisplus.annotation.IdType;
import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableId;
import com.bohuikeji.frame.module.admin.annotation.TableKeyField;
import com.bohuikeji.frame.module.admin.annotation.TableNewField;
import com.bohuikeji.frame.module.admin.annotation.TableQueryField;
import com.bohuikeji.frame.module.admin.annotation.TableShowField;
import com.bohuikeji.frame.module.admin.base.EntityBase;
import com.bohuikeji.frame.module.admin.entity.web.block.InputBlock;
import com.bohuikeji.frame.module.admin.entity.web.block.SelectBlock;
import com.bohuikeji.frame.module.admin.serializer.Decimal2Serializer;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.experimental.Accessors;

import java.math.BigDecimal;
import java.time.LocalDateTime;

/**
 * <p>
 * 客户钱包充值缴费记录
 * </p>
 *
 * @author 10552
 * @since 2019-12-27
 */
@Data
@EqualsAndHashCode(callSuper = true)
@Accessors(chain = true)
@ApiModel(value="TCustomerPay对象", description="客户钱包充值缴费记录")
public class TCustomerPay extends EntityBase<TCustomerPay> {

    private static final long serialVersionUID = 1L;

    @ApiModelProperty(value = "自增id")
    @TableId(value = "customer_pay_id", type = IdType.AUTO)
    @TableShowField(value = "ID",id = true)
    @TableNewField(show = false)
    private Integer customerPayId;

    @ApiModelProperty(value = "支付单号")
    @TableField("pay_num")
    @TableShowField("支付单号")
    @TableNewField(edit = false)
    @TableQueryField(show = false)
    private String payNum;

    @ApiModelProperty(value = "付款方式")
    @TableField("pay_way")
    @TableShowField("付款方式")
    @TableNewField(edit = false)
    @TableQueryField(like = false)
    private String payWay;

    @ApiModelProperty(value = "钱包Id")
    @TableField("wallet_id")
    private Integer walletId;

    @ApiModelProperty(value = "加盟商id")
    @TableField("fran_id")
    @TableShowField(value = "所属区域",show = true)
    @TableKeyField(table = "t_franchisee_information",keyShowFields = "fran_name",where = "fran_id in(##FRAN_IDS##)")
    @TableQueryField(like = false)
    @TableNewField(block = SelectBlock.class,edit = false,add = false)
    private Integer franId;


    @ApiModelProperty(value = "下单用户Id")
    @TableShowField("充值用户ID")
    @TableQueryField(show = false)
    @TableNewField(block = InputBlock.class,edit = false,add = false)
    @TableField("player_id")
    private Integer playerId;

    @ApiModelProperty(value = "预估支付总价")
    @TableField("pay_expect_price")
    @TableShowField(value = "支付预估金额(￥)",show = false)
    @TableNewField(edit = false)
    @JSONField(serializeUsing = Decimal2Serializer.class)
    private BigDecimal payExpectPrice=new BigDecimal("0.00").setScale(2, BigDecimal.ROUND_DOWN);

    @ApiModelProperty(value = "实际支付总价")
    @TableField("pay_fact_price")
    @TableShowField("实际支付金额(￥)")
    @TableNewField(edit = false)
    @JSONField(serializeUsing = Decimal2Serializer.class)
    private BigDecimal payFactPrice=new BigDecimal("0.00").setScale(2, BigDecimal.ROUND_DOWN);

    @ApiModelProperty(value = "支付状态(待付款,已支付,取消支付)")
    @TableField("pay_status")
    @TableShowField("支付状态")
    @TableNewField(edit = false)
    @TableQueryField(like = false)
    private String payStatus;

    @ApiModelProperty(value = "付款渠道")
    @TableField("pay_channel")
    @TableShowField(value = "付款渠道",show = false)
    @TableNewField(edit = false)
    @TableQueryField(like = false)
    private String payChannel;

    @ApiModelProperty(value = "渠道商单号(微信,支付宝，银联等第三方单号)")
    @TableField("pay_channel_num")
    @TableShowField("渠道商单号")
    @TableNewField(edit = false)
    @TableQueryField(show = false)
    private String payChannelNum;

    @ApiModelProperty(value = "交易创建时间")
    @TableField("create_time")
    @TableShowField(value = "交易发起时间",show = false)
    @TableNewField(edit = false)
    private LocalDateTime createTime;

    @ApiModelProperty(value = "支付时间")
    @TableField("pay_time")
    @TableShowField("支付时间")
    @TableNewField(edit = false)
    @TableQueryField(sort = 0,like = false)
    private LocalDateTime payTime;

    @ApiModelProperty(value = "支付场景")
    @TableField("pay_scenario")
    @TableShowField("支付场景")
    @TableNewField(edit = false)
    @TableQueryField(like = false)
    private String payScenario;


}
